
/*****************************************************************************/
/*                                                                           */
/*  THE HSEVAL HIGH SCHOOL TIMETABLE EVALUATOR                               */
/*  COPYRIGHT (C) 2009, Jeffrey H. Kingston                                  */
/*                                                                           */
/*  Jeffrey H. Kingston (jeff@it.usyd.edu.au)                                */
/*  School of Information Technologies                                       */
/*  The University of Sydney 2006                                            */
/*  AUSTRALIA                                                                */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify     */
/*  it under the terms of the GNU General Public License as published by     */
/*  the Free Software Foundation; either Version 3, or (at your option)      */
/*  any later version.                                                       */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful,          */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            */
/*  GNU General Public License for more details.                             */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License        */
/*  along with this program; if not, write to the Free Software              */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston MA 02111-1307 USA   */
/*                                                                           */
/*  FILE:         main.c                                                     */
/*  MODULE:       Main module, handles reading and operation dispatch        */
/*                                                                           */
/*****************************************************************************/
#define _POSIX_SOURCE
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/resource.h>
#include "externs.h"

#define DEBUG1 0
#define BannerFoot "HSEval Software Copyright Jeffrey H. Kingston 2012"


/*****************************************************************************/
/*                                                                           */
/*  HTML PageBegin()                                                         */
/*                                                                           */
/*  Begin an HSEVAL output page, returning the HTML handle where subsequent  */
/*  writes to that page should go.                                           */
/*                                                                           */
/*****************************************************************************/

HTML PageBegin(char *title)
{
  HTML html = NULL;
  html = HTMLPageBegin("./hseval.cgi", "HSEval home page", stdout);
  HTMLHeadBegin(html);
  if( title != NULL )
  {
    HTMLTitleBegin(html);
    HTMLText(html, title);
    HTMLTitleEnd(html);
  }
  HTMLHeadEnd(html);
  HTMLBodyBegin(html);
  HTMLSmallBegin(html);
  HTMLText(html, "HSEval");
  HTMLText(html, KheVersionBanner());
  HTMLSmallEnd(html);
  HTMLHorizontalRule(html);
  return html;
}


/*****************************************************************************/
/*                                                                           */
/*  void PageEnd(html)                                                       */
/*                                                                           */
/*  End an HSEVAL output page.                                               */
/*                                                                           */
/*****************************************************************************/

void PageEnd(HTML html)
{
  HTMLHorizontalRule(html);
  HTMLSmallBegin(html);
  HTMLText(html, BannerFoot);
  HTMLSmallEnd(html);
  HTMLBodyEnd(html);
  HTMLPageEnd(html);
}


/*****************************************************************************/
/*                                                                           */
/*  void Error(CGI cgi, char *text)                                          */
/*                                                                           */
/*  Report an error and exit.                                                */
/*                                                                           */
/*  The error text is to be taken literally, so any < and > chars in it      */
/*  must be encoded.                                                         */
/*                                                                           */
/*****************************************************************************/

void Error(CGI cgi, char *text)
{
  HTML html;

  html = PageBegin("HSEval: Error in uploaded XML file");
  HTMLBigHeading(html, "HSEval: Error in uploaded XML file");

  HTMLParagraphBegin(html);
  HTMLText(html, "The XML file you just uploaded has at least one error:");
  HTMLParagraphEnd(html);

  HTMLParagraphBegin(html);
  HTMLColouredBoxBegin(html, LightRed);
  HTMLLiteralText(html, text);
  HTMLColouredBoxEnd(html);
  HTMLParagraphEnd(html);

  HTMLParagraphBegin(html);
  HTMLText(html, "Return to the ");
  HTMLJumpFront(html);
  HTMLText(html, ".");
  HTMLParagraphEnd(html);

  PageEnd(html);
  CgiEnd(cgi);
  exit(0);
}


/*****************************************************************************/
/*                                                                           */
/*  void TimeOut(int sig)                                                    */
/*                                                                           */
/*  Signal handler function for when HSEval times out.                       */
/*                                                                           */
/*****************************************************************************/

static void TimeOut(int sig)
{
  HTML html;

  html = PageBegin("HSEval: Timeout");
  HTMLBigHeading(html, "HSEval: Timeout");

  HTMLParagraphBegin(html);
  HTMLText(html, "The call you have just made on HSEval was taking too");
  HTMLText(html, "long and has timed out.  Please do not use HSEval to");
  HTMLText(html, "evaluate very large archives, like the solutions to");
  HTMLText(html, "Part 2 of ITC2011.  Consult Jeff Kingston if you need");
  HTMLText(html, "to evaluate a very large archive.");
  HTMLParagraphEnd(html);

  HTMLParagraphBegin(html);
  HTMLText(html, "Return to the ");
  HTMLJumpFront(html);
  HTMLText(html, ".");
  HTMLParagraphEnd(html);

  PageEnd(html);
  exit(0);
}


/*****************************************************************************/
/*                                                                           */
/*  void CGIDisplayFrontPage(void)                                           */
/*                                                                           */
/*  Display the front page.                                                  */
/*                                                                           */
/*****************************************************************************/

static void CGIDisplayFrontPage(void)
{
  HTML html;
  /* struct rlimit rlim_rec;  int status; */
  
  html = PageBegin("The HSEval High School Timetable Evaluator");
  HTMLBigHeading(html, "The HSEval High School Timetable Evaluator");
  HTMLFormBeginUpload(html, true);

  HTMLParagraphBegin(html);
  HTMLColouredBoxBeginFullWidth(html, LightRed);
  HTMLText(html, "This version of HSEval supports the employee scheduling");
  HTMLText(html, "format XESTT as well as the high school timetabling format");
  HTMLText(html, "XHSTT.  Links to descriptions of both formats appear below.");
  HTMLText(html, "This version is based on a significantly upgraded version");
  HTMLText(html, "of KHE.  It should be faster, but please look out for bugs.");
  HTMLColouredBoxEnd(html);
  HTMLParagraphEnd(html);

  HTMLParagraphBegin(html);
  HTMLText(html, "This is the HSEval High School Timetable Evaluator.  It");
  HTMLText(html, "accepts an XML file in");
  HTMLJump1(html, "op", "spec", NULL, "XHSTT format");
  HTMLText(html, ", containing any number");
  HTMLText(html, "of instances of high school timetabling problems and groups");
  HTMLText(html, "of solutions to the instances, and evaluates the solutions.");
  HTMLText(html, "It also handles the");
  HTMLJump2(html, "op", "spec", "part", "employee", NULL,
    "XESTT employee scheduling format");
  HTMLText(html, ".  HSEval will time out if it consumes more than 5 minutes");
  HTMLText(html, "of CPU time, as is possible if you try to use it to");
  HTMLText(html, "evaluate very large archives (please don't).");
  HTMLParagraphEnd(html);

  HTMLParagraphBegin(html);
  HTMLText(html, "No data are stored on this site.  All operations are");
  HTMLText(html, "applied to a single XML file that you upload.  Specify");
  HTMLText(html, "the operation here:");
  HTMLRadioBoxesTabularBegin(html, "op", "summary_html");

  HTMLRadioBoxTabular(html, "report",
    "<b>XML Report.</b>\n"
    "Return a copy of the XML file with a report added to each solution,\n"
    "replacing any existing reports.");

  HTMLRadioBoxTabular(html, "summary_html",
    "<b>HTML Summary.</b>\n"
    "Return an HTML page containing tables summarizing the instances and\n"
    "solutions in the file.");

  HTMLRadioBoxTabular(html, "summary_latex",
    "<b>LaTeX Summary.</b>  Like the previous operation except that the\n"
    "tables are returned in LaTeX format.");

  HTMLRadioBoxTabular(html, "ranking_html",
    "<b>HTML Ranking.</b>\n"
    "Return an HTML page which ranks the solution groups in the file\n"
    "following the rules of the Third International Timetabling Competition.");

  HTMLRadioBoxTabular(html, "ranking_latex",
    "<b>LaTeX Ranking.</b>  Like the previous operation except that the\n"
    "tables are returned in LaTeX format.");

  HTMLRadioBoxTabular(html, "report_html",
    "<b>HTML Report.</b>\n"
    "Return an HTML page containing a detailed report on each solution.\n"
    "This is mainly useful when there are only a few solutions in the\n"
    "archive, since otherwise the page returned can be very long.");

  HTMLRadioBoxTabular(html, "timetables_html",
    "<b>HTML Timetables.</b>\n"
    "Return an HTML page containing, for each solution, a timetable for\n"
    "each resource, each with a table of violated constraints that apply\n"
    "to that resource.  Again, this page is mainly useful when there are\n"
    "only a few solutions; and it only works on instances that contain Day\n"
    "time subgroups.");

  HTMLRadioBoxTabular(html, "timetables_html_long",
    "<b>HTML Timetables (Long).</b>\n"
    "Like the previous operation, except that timetables for all event\n"
    "groups that have constraints are added.  This page is very long\n"
    "and is useful mainly for checking HSEval's report on a single solution.");

  HTMLRadioBoxTabular(html, "planning_html",
    "<b>HTML Planning Timetables.</b>\n"
    "Return an HTML page containing, for each solution, one planning\n"
    "timetable for each resource type of its instance.  A planning\n"
    "timetable is a single large table with one column per time and\n"
    "one row per resource.");

  HTMLRadioBoxTabular(html, "planning_html_highlit",
    "<b>HTML Planning Timetables (Highlighted).</b>\n"
    "Like the previous operation, except that clashes and split and partial\n"
    "resource assignments are highlighted.");

  HTMLRadioBoxesTabularEnd(html);
  HTMLText(html, "Specify the XML file here, then press Submit.  If the file");
  HTMLText(html, "is not in the right format, an HTML page will be returned");
  HTMLText(html, "giving the point of the first error.");
  HTMLParagraphEnd(html);

  HTMLParagraphBegin(html);
  HTMLFileUploadBox(html, "file");
  HTMLParagraphEnd(html);

  HTMLSubmitButton(html, "Submit");
  HTMLParagraphEnd(html);

  HTMLParagraphBegin(html);
  HTMLText(html, "HSEval is part of a wider project devoted to making");
  HTMLText(html, "instances and solutions of high school timetabling");
  HTMLText(html, "problems from around the world available in a standard");
  HTMLText(html, "format.  For further information about this project,");
  HTMLText(html, "consult");
  HTMLJumpToDocument(html, "http://www.utwente.nl/ewi/dmmp/gerhardpost/",
    NULL, "Gerhard Post");
  HTMLText(html, "'s");
  HTMLJumpToDocument(html, "http://www.utwente.nl/ctit/hstt/",
    NULL, "XHSTT web page");
  HTMLText(html, ".  HSEval itself was written and is maintained by");
  HTMLJumpToDocument(html, "http://jeffreykingston.id.au", NULL,
    "Jeffrey H. Kingston");
  HTMLText(html, ".  It uses his ");
  HTMLJumpToDocument(html, "http://jeffreykingston.id.au/khe",
    NULL, "KHE high school timetabing engine");
  HTMLText(html, ", a free, open source C library for reading and solving");
  HTMLText(html, "high school timetabling and employee scheduling problems.");
  HTMLText(html, "This web site is hosted on a server maintained by a web");
  HTMLText(html, "hosting company and paid for by Jeffrey H. Kingston.");
  HTMLJump1(html, "op", "timeout", NULL, "[timeout test]");
  HTMLParagraphEnd(html);

  HTMLFormEnd(html);

  /* ***
  HTMLParagraphBegin(html);
  getrlimit(RLIMIT_CORE, &rlim_rec);
  rlim_rec.rlim_cur = rlim_rec.rlim_max;
  status = setrlimit(RLIMIT_CORE, &rlim_rec);
  HTMLTextWithInt(html, "Core rlim_cur: %lu.", rlim_rec.rlim_cur);
  HTMLTextWithInt(html, "Core rlim_max: %lu.", rlim_rec.rlim_max);
  HTMLTextWithInt(html, "RLIM_INFINITY: %llu.", RLIM_INFINITY);
  HTMLTextWithInt(html, "sizeof(rlim_rec.rlim_max): %d.",
    sizeof(rlim_rec.rlim_max));
  HTMLTextWithInt(html, "status: %d.", status);
  HTMLParagraphEnd(html);
  *** */

  PageEnd(html);
}


/*****************************************************************************/
/*                                                                           */
/*  void CGISpec(CGI cgi)                                                    */
/*                                                                           */
/*  Print one section of the specification of the XML file format.           */
/*                                                                           */
/*****************************************************************************/

static void CGISpec(CGI cgi)
{
  char *op, *val;
  op = CgiNextOp(cgi);
  if( op == NULL || strcmp(op, "part") != 0 )
    CGISpecMain();
  else
  {
    val = CgiNextOneLineVal(cgi);
    if( val == NULL || strcmp(val, "main") == 0 )
      CGISpecMain();
    else if( strcmp(val, "constraints") == 0 )
      ConstraintSpecFull();
    else if( strcmp(val, "employee") == 0 )
      EmployeeSpecFull();
    else if( strcmp(val, "glossary") == 0 )
      CGISpecGlossary();
    else
      Error(cgi, "request to display unknown part of the specification");
  }
}


/*****************************************************************************/
/*                                                                           */
/*  void CGIVerify(void)                                                     */
/*                                                                           */
/*  Execute the Verify CGI operation.                                        */
/*                                                                           */
/*****************************************************************************/

/* ***
static void CGIVerify(CGI cgi)
{
  char *str;  XML_ELT archive_elt;  ARCHIVE archive;

  ** read and verify the file **
  if( !CGIRetrieve(cgi, "file", &str) || str[0] == '\0' )
    Error(cgi, "uploaded file was empty");
  XMLSetErrorHandler(&Error);
  archive_elt = XMLReadString(str);
  archive = BuildArchive(archive_elt, "hseval");
  
  ** if we get this far, everything must be all right **
  CGISuccess();
}
*** */


/*****************************************************************************/
/*                                                                           */
/*  bool SolnGroupContainsInvalidSolns(KHE_SOLN_GROUP soln_group)            */
/*                                                                           */
/*  Return true if soln_group contains at least one invalid solution.        */
/*                                                                           */
/*****************************************************************************/

static bool SolnGroupContainsInvalidSolns(KHE_SOLN_GROUP soln_group)
{
  int i;  KHE_SOLN soln;
  for( i = 0;  i < KheSolnGroupSolnCount(soln_group);  i++ )
  {
    soln = KheSolnGroupSoln(soln_group, i);
    if( KheSolnIsInvalid(soln) )
      return true;
  }
  return false;
}


/*****************************************************************************/
/*                                                                           */
/*  bool ArchiveContainsInvalidSolns(KHE_ARCHIVE archive)                    */
/*                                                                           */
/*  Return true if archive contains at least one invalid solution.           */
/*                                                                           */
/*****************************************************************************/

bool ArchiveContainsInvalidSolns(KHE_ARCHIVE archive)
{
  KHE_SOLN_GROUP soln_group;  int i;
  for( i = 0;  i < KheArchiveSolnGroupCount(archive);  i++ )
  {
    soln_group = KheArchiveSolnGroup(archive, i);
    if( SolnGroupContainsInvalidSolns(soln_group) )
      return true;
  }
  return false;
}


/*****************************************************************************/
/*                                                                           */
/*  void ArchiveReportInvalidSolns(KHE_ARCHIVE archive, HTML html)           */
/*                                                                           */
/*  Report on invalid solutions.                                             */
/*                                                                           */
/*****************************************************************************/

void ArchiveReportInvalidSolns(KHE_ARCHIVE archive, HTML html)
{
  KHE_SOLN_GROUP soln_group;  KHE_SOLN soln;  int i, j;  char buff[500];
  KML_ERROR ke;
  if( ArchiveContainsInvalidSolns(archive) )
  {
    /* header and explanatory paragraph */
    HTMLParagraphBegin(html);
    HTMLHeading(html, "Invalid solutions");
    HTMLParagraphEnd(html);

    HTMLParagraphBegin(html);
    HTMLText(html, "For each invalid solution, this listing shows its");
    HTMLText(html, "index within its solution group and the position and");
    HTMLText(html, "nature of its first error.");
    HTMLParagraphEnd(html);

    /* one paragraph for each solution group with an invalid soln */
    for( i = 0;  i < KheArchiveSolnGroupCount(archive);  i++ )
    {
      soln_group = KheArchiveSolnGroup(archive, i);
      if( SolnGroupContainsInvalidSolns(soln_group) )
      {
	/* soln group header */
	HTMLParagraphBegin(html);
	HTMLBoldBegin(html);
	HTMLText(html, "In solution group");
	HTMLTextNoBreak(html, KheSolnGroupId(soln_group));
	HTMLText(html, ":");
	HTMLBoldEnd(html);

	/* one line for each invalid soln */
	for( j = 0;  j < KheSolnGroupSolnCount(soln_group);  j++ )
	{
	  soln = KheSolnGroupSoln(soln_group, j);
	  if( KheSolnIsInvalid(soln) )
	  {
	    HTMLNewLine(html);
	    ke = KheSolnInvalidError(soln);
	    snprintf(buff, 500, "Solution %d (line %d col %d: %s)", j + 1,
	      KmlErrorLineNum(ke), KmlErrorColNum(ke), KmlErrorString(ke));
	    HTMLText(html, buff);
	  }
	}
	HTMLParagraphEnd(html);
      }
    }
  }
}


/*****************************************************************************/
/*                                                                           */
/*  KHE_ARCHIVE CGIReadAndVerifyArchive(CGI cgi, bool allow_invalid_solns,   */
/*    bool placeholder_solns)                                                */
/*                                                                           */
/*  If this function returns at all, its return value is the archive         */
/*  stored in cgi, assumed to be the next op and value.                      */
/*                                                                           */
/*  If allow_invalid_solns is true, some of the solutions may be invalid.    */
/*  If placeholder_solns is true, all solutions will be placeholder solns.   */
/*                                                                           */
/*****************************************************************************/

static void soln_fn(KHE_SOLN soln, void *impl)
{
  if( !KheSolnIsPlaceholder(soln) )
    KheSolnReduceToPlaceholder(soln);
}

KHE_ARCHIVE CGIReadAndVerifyArchive(CGI cgi, bool allow_invalid_solns,
  bool placeholder_solns, HA_ARENA_SET as)
{
  char *op;  KHE_ARCHIVE archive;  KML_ERROR ke;
  op = CgiNextOp(cgi);
  if( op == NULL || strcmp(op, "file") != 0 )
    Error(cgi, "uploaded file expected but missing");
  if( !KheArchiveReadIncremental(stdin, as, &archive, &ke, false, false,
	allow_invalid_solns, CgiRerunFile(cgi), NULL, NULL, NULL,
	NULL, placeholder_solns ? soln_fn : NULL, NULL) )
  {
    char buff[500];
    sprintf(buff, "line %d col %d: %s", KmlErrorLineNum(ke),
      KmlErrorColNum(ke), KmlErrorString(ke));
    Error(cgi, buff);
  }
  return archive;
}


/*****************************************************************************/
/*                                                                           */
/*  void CGIReport(CGI cgi, bool txt_suffix)                                 */
/*                                                                           */
/*  Execute the Report CGI operation.  The result file is to have ".xml"     */
/*  for its suffix unless txt_suffix is true, in which case it has ".txt".   */
/*                                                                           */
/*****************************************************************************/

static void CGIReport(CGI cgi, bool txt_suffix, HA_ARENA_SET as)
{
  KHE_ARCHIVE archive;
  archive = CGIReadAndVerifyArchive(cgi, false, false, as);
  fprintf(stdout, "Content-Type: text/%s\n", txt_suffix ? "plain" : "xml");
  fprintf(stdout, "Content-Disposition: attachment; filename=hseval.%s\n\n",
    txt_suffix ? "txt" : "xml");
  KheArchiveWrite(archive, true, stdout);
}


/*****************************************************************************/
/*                                                                           */
/*  void CGISummary(CGI cgi, PRINT_FORMAT print_format)                      */
/*                                                                           */
/*  Execute the Table CGI operation, making a table in format print_format.  */
/*                                                                           */
/*****************************************************************************/

static void CGISummary(CGI cgi, PRINT_FORMAT print_format, HA_ARENA_SET as)
{
  KHE_ARCHIVE archive;
  archive = CGIReadAndVerifyArchive(cgi, true, true, as);
  SummaryEvalPrint(archive, print_format, as);
}


/*****************************************************************************/
/*                                                                           */
/*  void CGIRanking(CGI cgi, PRINT_FORMAT print_format)                      */
/*                                                                           */
/*  Execute the Ranking CGI operation, with the result in print_format.      */
/*                                                                           */
/*****************************************************************************/

static void CGIRanking(CGI cgi, PRINT_FORMAT print_format, HA_ARENA_SET as)
{
  KHE_ARCHIVE archive;
  archive = CGIReadAndVerifyArchive(cgi, true, true, as);
  RankingEvalPrint(archive, print_format, as);
}


/*****************************************************************************/
/*                                                                           */
/*  void CGIReportHTML(CGI cgi)                                              */
/*                                                                           */
/*  Return a HTML page containing a detailed report on each solution.        */
/*                                                                           */
/*****************************************************************************/

static void CGIReportHTML(CGI cgi, HA_ARENA_SET as)
{
  KHE_ARCHIVE archive;  HTML html;  char buff[100];
  archive = CGIReadAndVerifyArchive(cgi, true, false, as);
  snprintf(buff, 100, "%s Evaluations Summary",
    KheArchiveId(archive) != NULL && strlen(KheArchiveId(archive)) < 60 ?
    KheArchiveId(archive) : "HSeval");
  html = PageBegin(buff);
  HTMLBigHeading(html, buff);

  HTMLParagraphBegin(html);
  HTMLText(html, "This page contains detailed evaluations of the");
  HTMLText(html, "solutions in the uploaded XML archive.");
  HTMLParagraphEnd(html);

  ArchiveReportHTML(archive, html, as);

  HTMLParagraphBegin(html);
  HTMLText(html, "Return to the ");
  HTMLJumpFront(html);
  HTMLText(html, ".");
  HTMLParagraphEnd(html);
  PageEnd(html);
}


/*****************************************************************************/
/*                                                                           */
/*  void CGITimetablesHTML(CGI cgi, bool with_event_groups)                  */
/*                                                                           */
/*  Return an HTML page containing timetables for each solution.             */
/*  There is always a timetable for each resource; if with_event_groups      */
/*  is true, there is a timetable for each constrained event group as well.  */
/*                                                                           */
/*****************************************************************************/

static void CGITimetablesHTML(CGI cgi, bool with_event_groups, HA_ARENA_SET as)
{
  KHE_ARCHIVE archive;  HTML html;  char buff[100];
  archive = CGIReadAndVerifyArchive(cgi, true, false, as);
  snprintf(buff, 100, "%s Solution Timetables",
    KheArchiveId(archive) != NULL && strlen(KheArchiveId(archive)) < 60 ?
    KheArchiveId(archive) : "HSeval");
  html = PageBegin(buff);
  HTMLBigHeading(html, buff);

  HTMLParagraphBegin(html);
  HTMLText(html, "For each solution in the uploaded XML archive, this page");
  HTMLText(html, "contains a timetable for each resource of the");
  HTMLText(html, "corresponding instance.  If any of the constraints");
  HTMLText(html, "that apply to that resource are violated, a table of");
  HTMLText(html, "those violations is shown below the timetable.");
  HTMLParagraphEnd(html);

  if( with_event_groups )
  {
    HTMLParagraphBegin(html);
    HTMLText(html, "Similarly, a timetable is shown for each event group of");
    HTMLText(html, "the instance to which at least one constraint applies;");
    HTMLText(html, "and if any of those constraints are violated, a table");
    HTMLText(html, "of those violations is shown.");
    HTMLParagraphEnd(html);
  }

  ArchiveTimetablesHTML(archive, with_event_groups, html, as);

  HTMLParagraphBegin(html);
  HTMLText(html, "Return to the ");
  HTMLJumpFront(html);
  HTMLText(html, ".");
  HTMLParagraphEnd(html);
  PageEnd(html);
}


/*****************************************************************************/
/*                                                                           */
/*  void CGIPlanning TimetablesHTML(CGI cgi, bool highlight_splits)          */
/*                                                                           */
/*  Print planning timetables for the solutions of the archive.              */
/*                                                                           */
/*****************************************************************************/

/* ***
static void CGIPlann ingTimetablesHTML(CGI cgi, bool highlight_splits)
{
  KHE_ARCHIVE archive;  HTML html;  char buff[100];
  archive = CGIReadAndVerifyArchive(cgi, true, false);
  snprintf(buff, 100, "%s Planning Timetables",
    KheArchiveId(archive) != NULL && strlen(KheArchiveId(archive)) < 60 ?
    KheArchiveId(archive) : "HSeval");
  html = PageBegin(buff);
  HTMLBigHeading(html, buff);

  HTMLParagraphBegin(html);
  HTMLText(html, "For each solution in the uploaded XML archive, this page");
  HTMLText(html, "contains one planning timetable for each resource type");
  HTMLText(html, "of the corresponding instance.  The Avai l column, where");
  HTMLText(html, "present, shows the remaining unused workload, when the");
  HTMLText(html, "the resource has a required limit workload constraint.");
  HTMLText(html, "In Unassigned rows it shows the unassigned workload.");
  HTMLText(html, "Grey boxes denote required unavailable times.");
  HTMLText(html, "Italic font denotes a missing, partial, or split");
  HTMLText(html, "assignment, whether or not an avoid split assignments");
  HTMLText(html, "constraint applies.  The times are reordered heuristically");
  HTMLText(html, "to bring split events together.");
  HTMLParagraphEnd(html);

  ArchivePlanningTimetablesHTML(archive, highlight_splits, html);

  HTMLParagraphBegin(html);
  HTMLText(html, "Return to the ");
  HTMLJumpFront(html);
  HTMLText(html, ".");
  HTMLParagraphEnd(html);
  PageEnd(html);
}
*** */


/*****************************************************************************/
/*                                                                           */
/*  void Run(void)                                                           */
/*                                                                           */
/*  Run this program as a CGI script.                                        */
/*                                                                           */
/*****************************************************************************/

static void Run(HA_ARENA_SET as)
{
  CGI cgi;  char *op, *val;  char buff[200];
  if( DEBUG1 )
    fprintf(stderr, "[ Run()\n");
  cgi = CgiBegin(HSEVAL_RERUN);
  op = CgiNextOp(cgi);
  if( op == NULL || strcmp(op, "op") != 0 )
    CGIDisplayFrontPage();
  else
  {
    val = CgiNextOneLineVal(cgi);
    if( val == NULL || strcmp(val, "display") == 0 )
      CGIDisplayFrontPage();
    else if( strcmp(val, "spec") == 0 )
      CGISpec(cgi);
    else if( strcmp(val, "timeout") == 0 )
      raise(SIGXCPU);

    /* these are the main options from the bullet list on the front page */
    else if( strcmp(val, "report") == 0 )
      CGIReport(cgi, false, as);
    /* *** no longer offering this option
    else if( strcmp(val, "freport") == 0 )
      CGIReport(cgi, true);
    *** */
    else if( strcmp(val, "summary_html") == 0 )
      CGISummary(cgi, PRINT_HTML, as);
    else if( strcmp(val, "summary_latex") == 0 )
      CGISummary(cgi, PRINT_LATEX, as);
    else if( strcmp(val, "ranking_html") == 0 )
      CGIRanking(cgi, PRINT_HTML, as);
    else if( strcmp(val, "ranking_latex") == 0 )
      CGIRanking(cgi, PRINT_LATEX, as);
    else if( strcmp(val, "report_html") == 0 )
      CGIReportHTML(cgi, as);
    else if( strcmp(val, "timetables_html") == 0 )
      CGITimetablesHTML(cgi, false, as);
    else if( strcmp(val, "timetables_html_long") == 0 )
      CGITimetablesHTML(cgi, true, as);
    else if( strcmp(val, "planning_html") == 0 )
      CGIPlanningTimetablesHTML(cgi, false, as);
    else if( strcmp(val, "planning_html_highlit") == 0 )
      CGIPlanningTimetablesHTML2(cgi, true, as);
    else
    {
      snprintf(buff, 200, "unknown CGI operation \"%s\" requested", val);
      Error(cgi, buff);
    }
  }
  CgiEnd(cgi);
  if( DEBUG1 )
    fprintf(stderr, "] Run()\n");
}


/*****************************************************************************/
/*                                                                           */
/*  void Help(int indent, FILE *fp)                                          */
/*                                                                           */
/*  Print a help message with the given indent onto fp.                      */
/*                                                                           */
/*****************************************************************************/
#define p(str) fprintf(fp, "%*s%s\n", indent, "", str)
#define p2(str1, str2) fprintf(fp, "%*s%s%s\n", indent, "", str1, str2)

static void Help(int indent, FILE *fp)
{
  p("");
  p("HSEval Usage");
  p("");
  p("With no command line arguments this executable runs as a CGI script,");
  p("reading from stdin or the QUERY_STRING environment variable in");
  p("the usual way, and printing an XML file or HTML page to stdout as");
  p("appropriate.  With command line arguments it behaves like this:");
  p("");
  p("  hseval -r <archive>");
  p("");
  p("    Read <archive>, check that it is legal XML, delete any");
  p("    existing reports, add a report generated by hseval to");
  p("    each solution, and write the resulting modified version");
  p("    of the original XML file to standard output.");
  p("");
  p("  hseval -t<options> <archive>");
  p("");
  p("    Read <archive>, check that it is legal XML, and produce a");
  p("    tabular summary of its contents, depending on <options>,");
  p("    which is a sequence of these characters in any order:");
  p("");
  p("    h     Produce a table in HTML format");
  p("    l     Produce a table in LaTeX format");
  p("    c     Include solution cost columns in the table");
  p("    t     Include running time columns in the table");
  p("    s     Show costs as integer soft costs only");
  p("    a     Add an Average row at the foot of the table");
  p("");
  p("  hseval -u");
  p("");
  p("    Print this usage message.");
  p("");
  p("  hseval -v");
  p("");
  p("    Print version, copyright, author, and license information.");
  p("");
  p("Here <archive> is an XML timetable archive file, beginning with");
  p("either <HighSchoolTimetableArchive> or <EmployeeScheduleArchive>,");
  p("optionally followed by these options:");
  p("");
  p("  -x<id>{,<id>}              Delete instances with these Ids");
  p("  -i<id>{,<id>}              Include only instances with these Ids");
  p("  -X<id>{,<id>}              Delete solution groups with these Ids");
  p("  -I<id>{,<id>}              Include only solution groups with these Ids");
  p("");
  p("Options -x and -i may not be used together.  Options -X and -I may not");
  p("be used together.  As a special case, -X without ids means delete all");
  p("solution groups.  These modifications occur only in the in-memory copy;");
  p("they do not affect the original file.  They are done before the archive");
  p("is used for anything else (reported on, tabularized, etc.).");
}


/*****************************************************************************/
/*                                                                           */
/*  void Version(int indent, FILE *fp)                                       */
/*                                                                           */
/*  Print version, copright, author, and license information.                */
/*                                                                           */
/*****************************************************************************/

static void Version(int indent, FILE *fp)
{
  p("");
  p2("HSEval ", KheVersionBanner());
  p("COPYRIGHT (C) 2009, Jeffrey H. Kingston");
  p("");
  p("Jeffrey H. Kingston (jeff@it.usyd.edu.au)");
  p("School of Information Technologies");
  p("The University of Sydney 2006");
  p("Australia");
  p("");
  p("This program is free software; you can redistribute it and/or");
  p("modify it under the terms of the GNU General Public License as");
  p("published by the Free Software Foundation; either Version 3, or");
  p("(at your option) any later version.");
  p("");
  p("This program is distributed in the hope that it will be useful,");
  p("but WITHOUT ANY WARRANTY; without even the implied warranty of");
  p("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the");
  p("GNU General Public License for more details.");
  p("");
  p("You should have received a copy of the GNU General Public License");
  p("with this program; if not, write to the Free Software Foundation,");
  p("Inc., 59 Temple Place, Suite 330, Boston MA 02111-1307 USA.");
  p("");
}


/*****************************************************************************/
/*                                                                           */
/*  FILE *OpenFile(char *fname)                                              */
/*                                                                           */
/*  Open file, possibly use stdin, possibly abort.                           */
/*                                                                           */
/*****************************************************************************/

/* *** no longer used
static FILE *OpenFile(char *fname)
{
  FILE *fp;
  if( fname == NULL )
  {
    ** no file name means read from stdin **
    fp = stdin;
  }
  else
  {
    ** read from fname **
    fp = fopen(fname, "r");
    if( fp == NULL )
    {
      fprintf(stderr, "hseval: cannot open file \"%s\" for reading\n", fname);
      exit(1);
    }
  }
  return fp;
}
*** */


/*****************************************************************************/
/*                                                                           */
/*  void WriteTable(KHE_ARCHIVE archive, char *options, HA_ARENA_SET as)     */
/*                                                                           */
/*  Write a tabular summary of archive according to options.                 */
/*                                                                           */
/*****************************************************************************/

static void WriteTable(KHE_ARCHIVE archive, char *options, HA_ARENA_SET as)
{
  PRINT_FORMAT fmt;  bool show_costs, show_times, show_averages, non_empty;
  bool soft_costs_only;  TABLE table;  HTML html;  char *p;  HA_ARENA a;

  /* get the options */
  fmt = PRINT_LATEX;
  show_costs = show_times = soft_costs_only = show_averages = false;
  for( p = options;  *p != '\0';  p++ ) switch( *p )
  {
    case 'h':

      fmt = PRINT_HTML;
      break;

    case 'l':

      fmt = PRINT_LATEX;
      break;

    case 'c':

      show_costs = true;
      break;

    case 't':

      show_times = true;
      break;

    case 's':

      soft_costs_only = true;
      break;

    case 'a':

      show_averages = true;
      break;

    default:

      HnFatal("HSEval:  unknown -t option \'%c\'", *p);
      break;
  }

  /* generate the table */
  a = HaArenaSetArenaBegin(as);
  /* a = HaAren aMake(); */
  table = EvalTableBuild(archive, show_costs, soft_costs_only, show_times,
    show_averages, &non_empty, a);
  /* ***
  if( show_times )
    table = RunningTimeTableBuild(archive, show_averages, &non_empty);
  else if( show_costs )
    table = EvalTableBuild(archive, soft_costs_only, show_averages);
  else
    HnFatal("HSEval:  in -t, neither c nor t option specified");
  *** */

  /* and print it */
  switch( fmt )
  {
    case PRINT_HTML:

      html = PageBegin("");
      TablePrintHTML(table, html);
      break;

    case PRINT_LATEX:

      TablePrintLaTeX(table, stdout);
      break;

    default:

      HnAbort("HSEval internal error (fmt = %d)", fmt);
      break;
  }
  HaArenaSetArenaEnd(as, a);
  /* HaArenaD elete(a); */
}


/*****************************************************************************/
/*                                                                           */
/*  KHE_ARCHIVE ReadArchive(char *file_name)                                 */
/*                                                                           */
/*  Read archive file file_name and return the archive.  Abort on error.     */
/*                                                                           */
/*****************************************************************************/

/* ***
static KHE_ARCHIVE ReadArchive(char *file_name)
{
  FILE *fp;  KHE_ARCHIVE res;  KML_ERROR ke;
  fp = OpenFile(file_name);
  if( fp == NULL )
    HnFatal("khe: cannot open archive file %s", file_name);
  if( !KheArchiveRead(fp, &res, &ke, false, false, false, NULL) )
    HnFatal("%s:%d:%d: %s", file_name, KmlErrorLineNum(ke),
      KmlErrorColNum(ke), KmlErrorString(ke));
  return res;
}
*** */


/*****************************************************************************/
/*                                                                           */
/*  Main program                                                             */
/*                                                                           */
/*****************************************************************************/

int main(int argc, char *argv[])
{
  KHE_ARCHIVE archive;  struct sigaction sig_act;  struct rlimit rlim_rec;
  int pos;  HA_ARENA a;  HA_ARENA_SET as;

  if( DEBUG1 )
    fprintf(stderr, "[ main(%d)\n", argc);
  a = HaArenaMake();
  as = HaArenaSetMake(a);
  if( argc == 1 )
  {
    /* signal handler for timeout */
    sigaction(SIGXCPU, NULL, &sig_act);
    sig_act.sa_handler = &TimeOut;
    sigaction(SIGXCPU, &sig_act, NULL);

    /* set time limit */
    getrlimit(RLIMIT_CPU, &rlim_rec);
    rlim_rec.rlim_cur = HSEVAL_TIME_LIMIT;
    setrlimit(RLIMIT_CPU, &rlim_rec);

    /* allow core dump */
    getrlimit(RLIMIT_CORE, &rlim_rec);
    rlim_rec.rlim_cur = rlim_rec.rlim_max;
    setrlimit(RLIMIT_CORE, &rlim_rec);

    /* run as CGI script if no command line options */
    Run(as);
  }
  else
  {
    /* the first command line option gives the operation */
    if( argv[1][0] != '-' )
      HnFatal("hseval: first option \"%s\" does not begin with \"-\"", argv[1]);
    switch( argv[1][1] )
    {
      case 'r':

	if( argc == 2 )
	  HnFatal("hseval: missing archive file name after -r");
	pos = 2;
        archive = KheArchiveReadFromCommandLine(argc, argv, &pos,
	  as, false, false, false, NULL);
	KheArchiveWrite(archive, true, stdout);
	break;

      case 't':

	if( argc == 2 )
	  HnFatal("hseval: missing archive file name after -t");
	pos = 2;
        archive = KheArchiveReadFromCommandLine(argc, argv, &pos,
	  as, false, false, false, NULL);
	if( pos != argc )
          HnFatal("hseval: unexpected command line argument \"%s\"", argv[pos]);
	WriteTable(archive, &argv[1][2], as);
	break;

      case 'u':

	Help(0, stderr);
	break;

      case 'v':

	Version(0, stderr);
	break;

      default:

	Help(0, stderr);
	fprintf(stderr, "\n");
	HnFatal("hseval: unknown first command line option \"%s\"", argv[1]);
	break;
    }
  }
  if( DEBUG1 )
    fprintf(stderr, "] main()\n");
  exit(0);
}
